"""Utility functions for reading and writing files."""

import base64
from typing import Dict, List, Union

import numpy as np

from mahautils.multics.simresults import SimResults


def decode_base64(base64_str: str) -> str:
    """Decodes Base64 binary data with UTF-8 encoding"""
    content_data = base64_str.split(',', maxsplit=1)[1]
    return base64.b64decode(content_data).decode('utf_8')


def load_simresults(dash_base64_contents: str, sim_results: SimResults) -> None:
    """Loads a simulation results file uploaded by Dash a :py:class:`dcc.Upload`
    object"""
    sim_results.clear()
    sim_results.set_contents(decode_base64(dash_base64_contents).split('\n'),
                             trailing_newline=True)
    sim_results.parse()


def sim_results_to_csv(sim_results: SimResults,
                       config: Dict[str, Dict[str, Union[bool, str]]]) -> str:
    """Saves simulation results data to a CSV file"""
    output_keys: List[str] = []
    output_units: List[str] = []
    output_data: List[np.ndarray] = []
    for key in config:
        export_settings = config.get(key, {'export': False})
        if not export_settings['export']:
            continue

        units = str(export_settings['units'])

        output_keys.append(key)
        output_units.append(units)

        if sim_results.num_time_steps > 0:
            output_data.append(sim_results.get_data(key, units))

    output_data_np = np.array(output_data).T

    # Create CSV file
    csv = ','.join(output_keys) + '\n'
    csv += ','.join(output_units) + '\n'

    if sim_results.num_time_steps > 0:
        for i in range(sim_results.num_time_steps):
            csv += ','.join([str(x) for x in output_data_np[i]]) + '\n'

    return csv



"""Page elements for uploading simulation results files."""

from typing import Dict, Union

# Mypy type checking disabled for packages that are not PEP 561-compliant
import dash  # type: ignore

from mahautils.multics.simresults import SimResults
from mahautils.multics.exceptions import SimResultsDataNotFoundError


def is_exportable(key: str, sim_results: SimResults):
    """Returns whether a simulation results variable has data associated with it
    in the simulation results file"""
    if sim_results.num_time_steps == 0:
        return True

    try:
        data = sim_results.get_data(key, sim_results.get_units(key))
    except SimResultsDataNotFoundError:
        return False

    return len(data) == sim_results.num_time_steps


def parse_sim_results_vars(sim_results: SimResults
                           ) -> Dict[str, Dict[str, Union[bool, str]]]:
    """Returns a dictionary containing all simulation results variables and
    their units from a Maha Multics simulation results file"""
    data: Dict[str, Dict[str, Union[bool, str]]] = {}

    for key in sim_results.variables:
        if is_exportable(key, sim_results):
            data[key] = {
                'export': True,
                'description': str(sim_results.get_description(key)),
                'units': sim_results.get_units(key),
            }

    return data


def upload_section():
    """Page elements to upload simulation results file"""
    return dash.html.Div([
        dash.html.H3('Step 1: Upload Simulation Results File'),
        dash.dcc.Upload(
            'Drag and Drop or Click to Browse',
            id='upload-data',
            style={
                'width': '30%',
                'height': '60px',
                'lineHeight': '60px',
                'borderWidth': '1px',
                'borderStyle': 'dashed',
                'borderRadius': '10px',
                'textAlign': 'center',
                'marginTop': '20px',
                'marginLeft': '0px',
                'marginRight': '0px',
            },
        ),
        dash.html.Div(
            dash.html.P(
                'Error: Unable to load simulation results file',
                id='upload-error-message',
                style={'color': 'red'},
            ),
            hidden=True,
            id='upload-error',
            style={'marginTop': '10px'},
        )
    ])



"""Code for generating the header of the simulation results exporter.
"""

# Mypy type checking disabled for packages that are not PEP 561-compliant
import dash  # type: ignore

from .constants import GUI_LONG_NAME, PROJECT_NAME


def app_header():
    """Creates a formatted header for the application"""

    return dash.html.Div([
        dash.html.H2(
            f'{PROJECT_NAME} {GUI_LONG_NAME}',
            style={
                'fontWeight': 'bold',
                'textAlign': 'center',
                'marginTop': '20px',
                'marginBottom': '30px',
            }),
    ])



"""Constants shared throughout the MahaUtils SimViewer GUI.
"""

# Importing the package version is a cylic import, but because the
# `__version__` variable is defined at the beginning of
# `mahautils/__init__.py`, it is cached and cyclic import does not
# cause problems
from mahautils import __version__ as VERSION  # pylint: disable=R0401  # noqa: F401


# GENERAL PROJECT SETTINGS ---------------------------------------------------
PROJECT_NAME = 'MahaUtils'
GUI_LONG_NAME = 'Simulation Results Exporter'
GUI_SHORT_NAME = 'SimExporter'

REPO_URL = 'https://github.com/nathan-hess/maha-research-utils'
